@@ -505,7 +505,7 @@ int main(int argc, char * argv[])
 {
 	FILE            *input_fh = stdin;
 	Dictionary      dict;
-	Sentence        sent;
+	Sentence        sent = NULL;
 	const char     *language="en";  /* default to english, and not locale */
 	int             pp_on=TRUE;
 	int             af_on=TRUE;
@@ -620,7 +620,9 @@ int main(int argc, char * argv[])
 	verbosity = parse_options_get_verbosity(opts);
 	check_winsize(opts);
 
-	prt_error("Info: Version %s. Enter \"!help\" for help.\n",
+	prt_error("Info: Dictionary version %s.\n",
+		linkgrammar_get_dict_version(dict));
+	prt_error("Info: Library version %s. Enter \"!help\" for help.\n",
 		linkgrammar_get_version());
 
 	/* Main input loop */
@@ -707,6 +709,15 @@ int main(int argc, char * argv[])
 			if (0 < num_linkages) printf("Got One !!!!!!!!!!!!!!!!!\n");
 		}
 
+		/* If asked to show bad linkages, then show them. */
+		if ((num_linkages == 0) && (!parse_options_get_batch_mode(opts)))
+		{
+			if (parse_options_get_display_bad(opts))
+			{
+				num_linkages = sentence_num_linkages_found(sent);
+			}
+		}
+
 		/* Now parse with null links */
 		if ((num_linkages == 0) && (!parse_options_get_batch_mode(opts)))
 		{
@@ -756,8 +767,12 @@ int main(int argc, char * argv[])
 		}
 
 		sentence_delete(sent);
+		sent = NULL;
 	}
 
+	/* Free stuff, so that mem-leak detectors don't commplain. */
+	sentence_delete(sent);
+
 	if (parse_options_get_batch_mode(opts))
 	{
 		/* print_time(opts, "Total"); */
